
# q-expansion of newform 425.2.j.b, downloaded from the LMFDB on 16 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 425
weight = 2
poly_data = [1, 0, 22, 0, 111, 0, 152, 0, 83, 0, 18, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, -27, 0, -75, 0, -16, 0, 34, 0, 15, 0, 1], 4], [[-6, -20, -38, -100, -1, -102, 35, -32, 15, -2, 1, 0], 4], [[-18, 0, -133, 0, -166, 0, -84, 0, -18, 0, -1, 0], 4], [[-6, 20, -38, 100, -1, 102, 35, 32, 15, 2, 1, 0], 4], [[13, 0, 125, 0, 188, 0, 98, 0, 19, 0, 1, 0], 4], [[0, 31, 0, 258, 0, 354, 0, 182, 0, 37, 0, 2], 4], [[5, 20, 74, 100, 223, 102, 183, 32, 50, 2, 3, 0], 4], [[0, -61, 0, -369, 0, -592, 0, -366, 0, -87, 0, -5], 4], [[3, 36, -25, 332, -86, 577, -66, 365, -17, 87, -1, 5], 4], [[3, -36, -25, -332, -86, -577, -66, -365, -17, -87, -1, -5], 4]]

hecke_ring_character_values = [[52, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [326, [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, -1, 0, 1, 0, -1, -1, 0, 1, 0, 0, 0], [-1, 1, 0, 0, 0, -1, -1, 1, 1, 0, 0, -1], [0, -3, -3, 1, 0, -1, 0, 0, 0, -1, 1, -1], [-2, 1, 0, -2, 0, 0, -1, 0, -1, 1, 0, -2], [0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 2, -2], [0, 1, 1, -1, -1, 1, 0, 0, -1, -1, -2, 0], [1, 1, 2, 0, 1, 1, -1, 1, -1, 1, -2, 0], [2, 1, -1, 2, 0, 1, 2, 2, -1, 0, 3, 0], [0, -1, -3, 0, 0, -3, -2, 0, -1, 0, 0, 0], [-2, 0, 1, 0, 0, 2, 1, 2, 0, 0, 0, -2], [-2, 0, 0, 0, -3, 3, 2, 0, -3, 0, -1, -1], [0, -3, -5, 1, 0, -1, 0, 2, 0, -3, -1, 1], [0, 0, 0, -1, 0, -3, -4, 0, 2, 0, 2, 2], [0, 0, -4, 0, 0, 0, 0, 2, 0, -2, 0, 0], [4, -1, -1, 0, 1, 1, 0, -4, -1, -1, 0, 0], [0, -1, 1, 1, 0, -1, 0, 2, 0, 1, 1, -1], [3, -4, 0, 4, 1, -4, -4, 3, 4, 1, 1, 0], [6, -1, 0, 0, 3, 1, 1, -6, -1, -3, 0, 2], [2, 2, 5, 0, 1, 2, 3, -2, 2, -1, 0, 1], [4, 0, 0, 1, 1, 2, 0, 0, -1, 0, -1, -1], [-4, 0, 0, -1, -5, 0, 1, 0, -1, 0, 0, 0], [0, -3, -1, 0, 0, 3, 2, 0, -3, 0, 0, -4], [2, 0, 0, -1, 3, 0, 3, 0, -1, 0, 0, 0], [0, 3, 5, -2, 0, 2, 0, 8, 0, 3, 1, -1], [2, 2, 6, 0, -2, 3, 4, -2, 2, 2, 0, 2], [2, -4, -3, 0, 2, 4, 1, -2, -4, -2, 0, 2], [2, 3, 2, -2, 5, 3, 1, 2, -3, 5, 4, 0], [-2, 0, 0, 4, -1, 1, -4, 0, 3, 0, -3, -3], [-2, -2, -3, 2, -1, -2, 1, -2, 2, -1, -1, 0], [0, 1, 9, -3, 0, 3, 0, 0, 0, -1, -7, 7], [3, 0, -4, 2, 3, 0, 4, 3, 0, 3, 3, 0], [-2, 0, 0, -4, -4, 0, -2, 0, -4, 0, -3, -3], [0, 2, -4, -2, 0, 2, 0, 8, 0, 4, 4, -4], [0, -2, -2, 3, 0, -3, 0, 8, 0, 2, 4, -4], [-6, -3, -4, 3, 2, -3, 1, -6, 3, 2, -2, 0], [0, -3, -4, 0, 0, -2, -1, 0, -3, 0, 0, -2], [10, -3, -1, 0, 2, 3, 2, -10, -3, -2, 0, 2], [0, 2, -4, 0, 0, 0, 0, 12, 0, 8, 5, -5], [1, -1, -2, 0, -1, 3, -1, -1, -1, 1, 0, 4], [0, 0, 0, 2, -6, 4, 6, 0, -2, 0, -3, -3], [-2, -2, 1, 0, -2, -2, -3, -2, 2, -2, 2, 0], [-6, -4, -6, 2, -3, -4, 2, -6, 4, -3, 2, 0], [8, 4, 1, 4, 3, 4, 3, 8, -4, 3, 5, 0], [2, -1, 3, 0, 2, 3, 4, -2, -1, -2, 0, 5], [0, 0, 0, -5, -3, -2, 4, 0, -3, 0, -3, -3], [-2, 1, 2, -1, 0, 1, -1, -2, -1, 0, 4, 0], [0, -3, 4, -1, 0, 1, 0, -4, 0, -1, 0, 0], [-2, -4, 2, 0, -5, 2, 6, 2, -4, 5, 0, -4], [4, 0, 0, 0, 0, -4, -8, 0, 4, 0, -2, -2], [6, 9, 9, -6, 9, 9, 0, 6, -9, 9, 0, 0], [-2, 0, 0, -2, -6, -4, -2, 0, 2, 0, 3, 3], [4, 0, 0, 3, 3, -2, 2, 0, 5, 0, 1, 1], [-14, 0, 0, 2, -3, -3, 2, 0, 5, 0, -3, -3], [-4, 0, 3, -4, 2, 0, -3, -4, 0, 2, 0, 0], [2, 0, 0, 6, 6, 0, 0, 0, 6, 0, 2, 2], [4, -5, -10, 0, 7, -1, -5, -4, -5, -7, 0, 2], [0, 2, -8, -4, 0, 4, 0, -2, 0, -2, -3, 3], [6, -6, 3, 9, -1, -6, -9, 6, 6, -1, -2, 0], [0, 2, 0, -6, 0, 6, 0, 6, 0, 4, 0, 0], [0, 5, 1, 0, 0, 0, 0, 8, 0, 11, 7, -7], [7, 6, 6, 0, 1, 6, 0, 7, -6, 1, -3, 0], [14, 5, 0, 4, 1, 5, 5, 14, -5, 1, 6, 0], [2, -8, -11, 12, 0, -8, 3, 2, 8, 0, 6, 0], [0, 2, 0, -2, 0, 2, 0, -16, 0, -8, -2, 2], [-4, 1, 3, 0, 8, 3, 2, 4, 1, -8, 0, 6], [-2, 6, 3, -11, 5, 6, 3, -2, -6, 5, 0, 0], [0, 2, 10, -10, 0, 10, 0, -10, 0, -2, -5, 5], [0, 10, 8, -8, 0, 8, 0, -6, 0, 6, 2, -2], [0, -10, -18, 12, 0, -12, 0, 4, 0, -8, 3, -3], [6, 1, 2, -1, -2, 1, -1, 6, -1, -2, 4, 0], [0, 1, 1, -1, 0, 1, 0, -12, 0, 1, -3, 3], [-4, 3, 7, 0, 4, 3, -4, -4, -3, 4, -1, 0], [0, 0, 0, 5, -3, 2, 6, 0, 3, 0, 5, 5], [0, 1, 6, -5, 0, 5, 0, -10, 0, -5, -7, 7], [-10, 6, 5, -14, -2, 6, 1, -10, -6, -2, -4, 0], [5, -5, -12, 0, 1, -3, -7, -5, -5, -1, 0, 4], [2, 0, 0, -2, 2, 0, 14, 0, -2, 0, 6, 6], [-4, 1, -3, 0, 4, -1, -4, 4, 1, -4, 0, 5], [-2, 0, 0, 1, 9, 8, 5, 0, -7, 0, -1, -1], [-6, 0, -1, 0, -7, -10, -1, 6, 0, 7, 0, -7], [4, 0, 0, 5, -5, 6, 4, 0, -1, 0, 1, 1], [1, 5, 0, -10, -2, 5, 5, 1, -5, -2, 5, 0], [0, -15, -17, 6, 0, -6, 0, 8, 0, -1, 3, -3], [-6, 5, -1, 0, 1, -5, -6, 6, 5, -1, 0, 0], [-4, 0, 0, -3, -9, 6, -2, 0, -9, 0, -3, -3], [0, 10, 2, -2, 0, 2, 0, 2, 0, -10, -3, 3], [0, 7, -7, 2, 0, -2, 0, 16, 0, 3, 3, -3], [2, 0, 0, 0, 1, 1, 6, 0, -1, 0, 7, 7], [6, -9, -11, 8, -4, -9, 2, 6, 9, -4, 1, 0], [0, 3, 2, -9, -8, 3, 1, 0, -3, -8, -12, 0], [0, 2, -4, 2, 0, -2, 0, 2, 0, -4, -5, 5], [12, 1, 9, 0, 0, 3, 8, -12, 1, 0, 0, 3], [-6, -12, -13, 7, -9, -12, 1, -6, 12, -9, 2, 0], [0, 0, 0, -8, 2, 10, 2, 0, -18, 0, 0, 0], [3, 9, 14, 0, -3, 11, 5, -3, 9, 3, 0, 2], [0, 1, 13, -7, 0, 7, 0, -2, 0, -1, -7, 7], [-1, 0, -2, -2, -14, 0, 2, -1, 0, -14, -8, 0], [8, 7, 1, 0, -3, 6, -6, -8, 7, 3, 0, 6], [0, 1, 7, 3, 0, -3, 0, 0, 0, -5, 3, -3], [8, 0, 0, -9, 3, -4, -2, 0, -5, 0, -1, -1], [0, 4, -2, 2, 0, -2, 0, 10, 0, 4, 0, 0], [0, -4, 5, 0, -3, -2, 9, 0, -4, 3, 0, 3], [0, 3, 17, -5, 0, 5, 0, 0, 0, 3, -3, 3], [16, 0, 0, 3, 1, 2, 2, 0, 1, 0, 5, 5], [-8, 0, 0, 5, -10, 5, 2, 0, 0, 0, -10, -10], [-12, 0, 0, 2, 6, 4, 10, 0, -2, 0, -3, -3], [-4, -3, -9, 0, 1, -11, -6, 4, -3, -1, 0, 0], [6, -1, 9, 0, 5, -2, 10, -6, -1, -5, 0, -2], [0, -2, 6, 0, 0, 0, 0, 10, 0, 0, 10, -10], [-10, 5, 0, 0, -5, -1, -5, 10, 5, 5, 0, -12], [5, 10, 4, 0, -1, -4, -6, -5, 10, 1, 0, 5], [0, -10, -16, 6, 0, -6, 0, 8, 0, 4, 16, -16], [0, -5, -1, 4, -3, -5, -4, 0, 5, -3, 2, 0], [0, 4, 8, 0, -4, -5, 4, 0, 4, 4, 0, 0], [0, -15, -11, 11, 0, -11, 0, 2, 0, -3, -1, 1], [6, -1, -1, -2, 4, -1, 0, 6, 1, 4, 2, 0], [2, 0, 0, -4, 0, -4, 6, 0, 0, 0, 2, 2], [0, 4, 16, -4, 0, 4, 0, 2, 0, 12, 1, -1], [-6, -3, 4, -4, -3, -3, -7, -6, 3, -3, -4, 0], [0, -3, 0, 0, 3, -5, 3, 0, -3, -3, 0, -2], [-2, 11, 9, 0, -3, -10, -2, 2, 11, 3, 0, -4], [-1, -1, -2, 0, -8, -11, -1, 1, -1, 8, 0, 3], [12, 0, 0, -1, 3, -8, 3, 0, 7, 0, 13, 13], [11, -4, -4, 12, -8, -4, 0, 11, 4, -8, 4, 0], [5, -9, -10, 18, 4, -9, 1, 5, 9, 4, 9, 0], [0, -7, -29, 10, 0, -10, 0, 12, 0, -1, 3, -3], [-10, 0, 0, 2, 6, -6, -14, 0, 8, 0, -2, -2], [0, -2, 2, -4, 0, 4, 0, 10, 0, -4, 2, -2], [20, 3, 13, 0, 9, 2, 10, -20, 3, -9, 0, 20], [9, -4, -14, 4, 5, -4, 10, 9, 4, 5, 5, 0], [0, 0, 0, 3, -1, 2, -4, 0, 1, 0, -5, -5], [2, 0, 0, 13, -11, -4, 1, 0, 17, 0, -2, -2], [-4, 0, 0, -5, 5, 10, 11, 0, -15, 0, 0, 0], [0, 0, 0, -9, 1, -4, -12, 0, -5, 0, -5, -5], [2, -2, 5, 0, 7, 5, 7, -2, -2, -7, 0, -2], [-26, 0, 0, -4, -8, -4, -20, 0, 0, 0, -8, -8], [-1, 6, 0, 0, -12, -8, -6, 1, 6, 12, 0, -14], [1, 10, -2, 0, 1, 10, 12, 1, -10, 1, -5, 0], [8, 3, -1, -2, 7, 3, 4, 8, -3, 7, 2, 0], [-10, 2, -8, 1, -2, 2, 10, -10, -2, -2, 2, 0], [4, 4, 4, -13, 4, 4, 0, 4, -4, 4, 12, 0], [10, 0, 0, -10, 18, -8, -14, 0, -2, 0, 6, 6], [1, 0, -16, 0, 5, 2, -16, -1, 0, -5, 0, -1], [0, -9, -9, 0, -2, 3, 0, 0, -9, 2, 0, 0], [12, -1, -6, 14, 1, -1, 5, 12, 1, 1, 10, 0], [0, -6, -14, 2, 0, -2, 0, -6, 0, -16, -5, 5], [0, 3, 13, -7, 0, 7, 0, 2, 0, 5, -3, 3], [2, -5, 8, -2, -1, -5, -13, 2, 5, -1, -12, 0], [-8, -3, -3, 10, 11, -3, 0, -8, 3, 11, 14, 0], [0, -1, 3, 2, 0, -2, 0, 12, 0, 1, 9, -9], [-6, 6, 0, 0, 0, -5, -6, 6, 6, 0, 0, -2], [-12, -1, -10, 3, -8, -1, 9, -12, 1, -8, 0, 0], [-7, 13, 18, 0, -8, -5, 5, 7, 13, 8, 0, 5], [-12, 0, 0, -8, -2, 10, 2, 0, -18, 0, -7, -7], [6, -4, -3, 0, -4, 6, 1, -6, -4, 4, 0, 6], [-28, 0, 0, -5, 0, -3, -8, 0, -2, 0, -2, -2], [7, 0, 4, 0, -8, 12, 4, -7, 0, 8, 0, 2], [24, 1, 6, 0, 6, -4, 5, -24, 1, -6, 0, 6], [0, -1, -11, 3, 0, -3, 0, 0, 0, -11, -5, 5], [-30, 0, 0, -2, -1, -9, -6, 0, 7, 0, -1, -1], [0, 4, 18, -4, 0, 4, 0, -4, 0, -14, -11, 11], [-4, 0, 0, 9, -6, -21, -14, 0, 30, 0, -2, -2], [8, 8, -7, 0, 7, -7, -15, -8, 8, -7, 0, 0], [-11, -9, -10, 8, -14, -9, 1, -11, 9, -14, 5, 0], [4, 9, 11, -8, 0, 9, -2, 4, -9, 0, -2, 0]]
